@@ -30,14 +30,14 @@ class OID_Map final {
       static OID_Map& global_registry();
 
    private:
-      static std::unordered_map<std::string, std::string> load_oid2str_map();
+      static std::unordered_map<OID, std::string> load_oid2str_map();
       static std::unordered_map<std::string, OID> load_str2oid_map();
 
       OID_Map();
 
       mutex_type m_mutex;
       std::unordered_map<std::string, OID> m_str2oid;
-      std::unordered_map<std::string, std::string> m_oid2str;
+      std::unordered_map<OID, std::string> m_oid2str;
 };
 
 }  // namespace Botan
