@@ -12,10 +12,12 @@
 
 #if defined(BOTAN_HAS_RSA)
 
+   #include <botan/numthry.h>
+   #include <botan/p11_mechanism.h>
    #include <botan/pubkey.h>
    #include <botan/rng.h>
    #include <botan/internal/blinding.h>
-   #include <botan/internal/p11_mechanism.h>
+   #include <botan/internal/mod_inv.h>
    #include <botan/internal/pk_ops_impl.h>
 
 namespace Botan::PKCS11 {
@@ -114,11 +116,12 @@ class PKCS11_RSA_Decryption_Operation final : public PK_Ops::Decryption {
                                       RandomNumberGenerator& rng) :
             m_key(key),
             m_mechanism(MechanismWrapper::create_rsa_crypt_mechanism(padding)),
+            m_barrett_mod_n(Modular_Reducer::for_public_modulus(m_key.get_n())),
             m_blinder(
-               m_key.get_n(),
+               m_barrett_mod_n,
                rng,
                [this](const BigInt& k) { return power_mod(k, m_key.get_e(), m_key.get_n()); },
-               [this](const BigInt& k) { return inverse_mod(k, m_key.get_n()); }) {
+               [this](const BigInt& k) { return inverse_mod_rsa_public_modulus(k, m_key.get_n()); }) {
          m_bits = m_key.get_n().bits() - 1;
       }
 
@@ -158,6 +161,7 @@ class PKCS11_RSA_Decryption_Operation final : public PK_Ops::Decryption {
    private:
       const PKCS11_RSA_PrivateKey& m_key;
       MechanismWrapper m_mechanism;
+      Modular_Reducer m_barrett_mod_n;
       size_t m_bits = 0;
       Blinder m_blinder;
 };
